<?php

class shopBifitkassaPlugin extends shopPlugin
{
	const PLUGIN_ID = 'bifitkassa';
	const LOG_FILE_NAME = 'shop/plugins/bifitkassa/fiscalization.log';

	public function processOrderStatus($params)
	{
		// ORDER DATA

		/// get order data
		$order = $this->getOrder(ifset($params['order_id']));

		/// payment method
		$payment_id = $order->params['payment_id'];

		/// order status
		$status_id = ifset($params['action_id']);

		/// payment State
		$payed = !in_array($order->paid_date, ['-1', '']) ? 1 : null;

		// PLUGINS PARAMS

		/// selected operating mode
		$test = $this->getSettings('test') ? '-test' : '';

		/// connection token
		$token =
			$test != ''
				? 'P5cKbUUD9uSSrSlGdzspLblvBnD0GzTAE0cLmAPSEMxJ79DtLE'
				: $this->getSettings('token');

		// iterating over the document type tabs
		for ($i = 1; $i <= 5; $i++) {
			// will find out whether it is necessary to use this type of document
			$use_type = $this->getSettings('use_type_' . $i) ? true : false;

			// if need to use
			if ($use_type) {
				// get parameters
				/// order paid (yes / no / disregard) (2/1/0)
				$order_paid = $this->getSettings('order_paid_' . $i);
				/// selected payment method (s)
				$pm_ids = $this->getSettings('pm_ids_' . $i);
				/// selected order status (s)
				$pm_statuses = $this->getSettings('pm_statuses_' . $i);
				/// use refused
				$use_refused = $this->getSettings('lk_use_refused') ? 1 : null;
				/// connection tab
				if ($i == 5 && $use_refused) {
					//// selected refused status (s)
					$lk_refused_statuses = $this->getSettings(
						'lk_refused_statuses'
					);
					/// if in array merge status arrays
					if (in_array($status_id, $lk_refused_statuses)) {
						$pm_statuses = array_merge(
							$pm_statuses,
							$lk_refused_statuses
						);
					}
				}

				// If the payment method and order status match
				if (
					isset($pm_ids) &&
					isset($pm_statuses) &&
					in_array($payment_id, $pm_ids) &&
					in_array($status_id, $pm_statuses) &&
					$token != ''
				) {
					// if "order paid" is selected,
					// and it is NOT paid, then we check the following tab
					if ($order_paid == '2' && !$payed) {
						continue;
					}

					// if "order not paid" is selected,
					// and it is paid, then we check the next tab
					if ($order_paid == '1' && $payed) {
						continue;
					}

					// main processes
					if ($i == 5) {
						// get lk-data
						$lk = self::__getLkData($order, $i, $status_id);
						// update BIFIT-account data
						self::__saveOrder($lk);
					} else {
						// start BIFIT fiscal-process
						self::__mainProcess($order, $test, $token, $i);
					}
				}
			}
		}
	}

	public static function __createGUID($order)
	{
		// Create GUID (Globally Unique Identifier)
		$guid = '';
		$namespace = rand(11111, 99999);
		$uid = uniqid('', true);
		$data = $namespace;
		$data .= $_SERVER['REQUEST_TIME'];
		$data .= $_SERVER['HTTP_USER_AGENT'];
		$data .= $_SERVER['REMOTE_ADDR'];
		$data .= $_SERVER['REMOTE_PORT'];
		$data .= $order['number'];
		$data .= $order['summ'];
		$hash = strtoupper(hash('ripemd128', $uid . $guid . md5($data)));
		$guid =
			substr($hash, 0, 8) .
			'-' .
			substr($hash, 8, 4) .
			'-' .
			substr($hash, 12, 4) .
			'-' .
			substr($hash, 16, 4) .
			'-' .
			substr($hash, 20, 12);

		return strtolower($order['cms'] . '-' . $guid);
	}

	function __getLkData($order, $i, $status_id)
	{
		// plugin id
		$moduleId = self::PLUGIN_ID;
		// tax system
		$tax_system = $this->getSettings('tax_system_' . $i);
		// method of calculation
		$calc_method = $this->getSettings('calculation_method');
		// VAT
		$vat = $this->getSettings('vat');

		// user params
		$user_firstname = $order->contact->get('firstname', 'default');
		$user_lastname = $order->contact->get('lastname', 'default');
		$user_name = trim($user_lastname . ' ' . $user_firstname);
		$user_phone = self::__checkPhone(
			$order->contact->get('phone', 'default')
		);
		$user_email = self::__checkEmail(
			$order->contact->get('email', 'default')
		);
		$user_address = '';
		$address_arr = ['street', 'city', 'region', 'zip', 'country'];
		foreach ($address_arr as $val) {
			$addr = $order->contact->get('address:' . $val, 'default');
			if ($addr != '') {
				$user_address .= $user_address != '' ? ', ' . $addr : $addr;
			}
		}

		// user address on the receipt (phone or email)
		$address = $address_type == 'phone' ? $user_phone : $user_email;

		/// an array of products to fill
		$r_products = [];
		/// an array of order items
		$products = $order->items;

		// iterate over the array of order items
		foreach ($products as $key_id => $product) {
			/// adjusting the price of the product taking into account discounts
			$p_price = self::__formatprc(
				self::__checkProductPrice(
					$product['price'],
					$order->total + $order->total,
					$order->discount,
					0
				)
			);

			// we clear the name of the product from special characters
			$p_name = preg_replace(
				'/[^\p{L}0-9 ]/iu',
				' ',
				html_entity_decode($product['name'])
			);
			$p_name = preg_replace('/ +/', ' ', trim($p_name));

			// form an array of goods for the check
			$r_products[$key_id] = new stdClass();
			$r_products[$key_id]->product_id = $product['product_id'];
			$r_products[$key_id]->product_name = $p_name;
			$r_products[$key_id]->product_item_price = $p_price;
			$r_products[$key_id]->product_quantity = $product['quantity'];
			$r_products[$key_id]->product_tax = 0;
		}

		// lk data
		$lk = new stdClass();
		$lk->order_id = $order->id;
		$lk->order_number = (int) filter_var(
			$order->id_str,
			FILTER_SANITIZE_NUMBER_INT
		);
		$lk->order_price = $order->total;
		$lk->order_delivery_id = $order->params['shipping_id'];
		$lk->order_price_delivery = $order->shipping;
		$lk->order_status_id = $status_id;
		$lk->user_name = $user_firstname;
		$lk->user_last_name = $user_lastname;
		$lk->user_phone = $user_phone;
		$lk->user_address = $user_address;
		$lk->address = $address;
		$lk->r_products = $r_products;
		$lk->receipt_type = $receipt_type;
		$lk->tax_system = $tax_system;
		$lk->calc_method = $calc_method;
		$lk->vat = $vat;

		return $lk;
	}

	function __saveOrder($lk)
	{
		// Options
		$login = self::__checkPhone($this->getSettings('login'));
		$password = $this->getSettings('password');
		$organization_id = $this->getSettings('organization');
		$trade_object_id = $this->getSettings('trade_object');
		$use_refused = $this->getSettings('lk_use_refused') ? true : false;

		// Get token
		$tokenData = self::__getToken($login, $password);
		$token = $tokenData->access_token;

		// parameters for ordering
		$onlineOrderItems = '';
		$onlineOrderPayments = '';

		// form order data
		foreach ($lk->r_products as $key_id => $product) {
			// address for POST request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures/external_id/' .
				$product->product_id .
				'?organization_id=' .
				$organization_id;

			// link data
			$data = '';

			// headers
			$headers = array(
				'Content-Type: application/x-www-form-urlencoded',
				'Authorization: Bearer ' . $token
			);

			$prod = json_decode(self::__loadData($url, $headers, $data, 'GET'));

			if ($prod == null) {
				// CREATE A NEW PRODUCT

				// address for POST request
				$url =
					'https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures';

				// link data
				$data = '{';
				$data .= '"item": {';
				$data .=
					'"vendorCode": "' . $product->product_vendor_code . '", ';
				$data .= '"name":"' . $product->product_name . '", ';
				$data .= '"externalId":"' . $product->product_id . '", ';
				$data .= '"sellingPrice":' . $product->product_item_price;
				$data .= '},';
				$data .= '"organizationId":"' . $organization_id . '"';
				$data .= '}';

				// headers
				$headers = array(
					'Content-Type: application/json',
					'Authorization: Bearer ' . $token
				);

				// get data from request
				$tov = json_decode(self::__loadData($url, $headers, $data));
			} else {
				$tov = $prod->id;
			}

			// display product data in the order
			$onlineOrderItems .= $onlineOrderItems != '' ? ', ' : '';
			$onlineOrderItems .= '{';
			$onlineOrderItems .= '"nomenclatureId":"' . $tov . '", ';
			$onlineOrderItems .= '"paymentSubject":"PRODUCT", ';
			$onlineOrderItems .= '"name":"' . $product->product_name . '", ';
			$onlineOrderItems .=
				'"price":' .
				self::__formatprc($product->product_item_price) .
				', ';
			$onlineOrderItems .= '"quantity":' . $product->product_quantity;
			$onlineOrderItems .= '}';
		}

		// delivery details
		if ($lk->order_price_delivery > 0) {
			// shipping tax
			$shipment_tax_percent = 0;

			// display data of delivery method in order
			$onlineOrderPayments = '{';
			$onlineOrderPayments .=
				'"calculationMethod":"' . $lk->calc_method . '", ';
			$onlineOrderPayments .= '"paymentSubject":"SERVICE", ';
			$onlineOrderPayments .= '"name":"' . _wp('Доставка') . '", ';
			$onlineOrderPayments .=
				'"price":' .
				self::__formatprc($lk->order_price_delivery) .
				', ';
			$onlineOrderPayments .= '"quantity":1, ';
			$onlineOrderPayments .=
				$lk->vat_mode == 1
					? '"vat":"' . $lk->vat . '", '
					: '"vat":"VAT_' . floor($shipment_tax_percent) . '", ';
			$onlineOrderPayments .=
				'"total":' . self::__formatprc($lk->order_price_delivery);
			$onlineOrderPayments .= '}';
		}

		// formatting time
		$time = array_sum(explode(' ', microtime()));
		$time = str_replace('.', '', $time);
		$time = substr($time, 0, -1);

		// CREATE A USER IN THE CASH

		// address for POST request
		$url = 'https://kassa.bifit.com/cashdesk-api/v1/protected/clients';

		// headers
		$headers = array(
			'Content-Type: application/json',
			'Authorization: Bearer ' . $token
		);

		// link data
		$data = '{';
		$data .= '"organizationId":"' . $organization_id . '", ';
		$data .= '"phone":"' . $lk->user_phone . '", ';
		$data .= '"lastName":"' . trim($lk->user_last_name) . '", ';
		$data .= '"firstName":"' . trim($lk->user_name) . '", ';
		$data .= '"address":"' . $lk->address . '"';
		$data .= '}';

		// get a client
		$clientId = json_decode(self::__loadData($url, $headers, $data));

		// if there is a client
		if ($clientId->type == "ERROR") {
			// address for POST request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/clients/list/read?organization_id=' .
				$organization_id;

			// link data
			$data = false;

			// headers
			$headers = array(
				'Content-Type: application/json',
				'Authorization: Bearer ' . $token
			);

			// get all clients
			$clients = json_decode(self::__loadData($url, $headers, $data));
			foreach ($clients as $clientsItem) {
				if ($clientsItem->phone == $lk->user_phone) {
					$clientId = $clientsItem->id;
				}
			}
		}

		// SENDING AN ORDER TO THE CASHIER

		// checking order availability
		// address for GET request
		$url =
			'https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders/list/read/' .
			$clientId .
			'?organization_id=' .
			$organization_id;

		// link data
		$data = false;

		// headers
		$headers = array(
			'Content-Type: application/x-www-form-urlencoded',
			'Authorization: Bearer ' . $token
		);

		// get orders
		$clientOrders = json_decode(
			self::__loadData($url, $headers, $data, 'GET')
		);
		$receipt_id = 0;
		$del_type = '';
		$order_deleted = null;

		// search order by id
		foreach ($clientOrders as $key => $clientOrder) {
			if ($clientOrder->externalId == $lk->order_number) {
				$receipt_id = $clientOrder->id;
				$del_type = $clientOrder->deliveryType;
				break;
			}
		}

		// type of delivery
		$lk_courier_deliveries = $this->getSettings('lk_courier_delivery');
		$lk_self_deliveries = $this->getSettings('lk_self_delivery');
		$deliveryType = 'SERVICE';

		if (
			isset($lk_courier_deliveries) &&
			in_array($lk->order_delivery_id, $lk_courier_deliveries)
		) {
			$deliveryType = 'COURIER';
		} elseif (
			isset($lk_self_deliveries) &&
			in_array($lk->order_delivery_id, $lk_self_deliveries)
		) {
			$deliveryType = 'SELF';
		}

		// assign "is_update"
		$lk_refused_statuses = $this->getSettings('lk_refused_statuses');
		$is_update =
			$receipt_id != 0 &&
			isset($lk_refused_statuses) &&
			in_array($lk->order_status_id, $lk_refused_statuses) &&
			$use_refused
				? 1
				: null;

		// new order || "is_update" conditions
		if ($deliveryType != 'SERVICE' && ($receipt_id == 0 || $is_update)) {
			// address for request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders';

			// headers
			$headers = array(
				'Content-Type: application/json',
				'Authorization: Bearer ' . $token
			);

			// data
			if ($is_update) {
				$url .= '/' . $receipt_id . '/status';
				$type = 'PUT';
				$statusType = $del_type == 'SELF' ? 'REFUSED' : 'DELETED';
				// Basic data
				$data = '{';
				$data .= '"onlineOrderId":"' . $receipt_id . '",';
				$data .= '"statusType":"' . $statusType . '",';
				$data .= '"statusDate":0,';
				$data .= '"statusInfo":""';
				$data .= '}';
			} else {
				$type = 'POST';
				$statusType =
					$deliveryType == 'SELF'
						? 'ACCEPTED'
						: $this->getSettings('lk_new_courier_order_status');
				// Basic data
				$onlineOrder = '{';
				$onlineOrder .= '"organizationId":"' . $organization_id . '", ';
				$onlineOrder .= '"tradeObjectId":"' . $trade_object_id . '", ';
				$onlineOrder .= '"taxSystem":"' . $lk->tax_system . '", ';
				$onlineOrder .= '"externalId":"' . $lk->order_number . '", ';
				$onlineOrder .= '"deliveryType":"' . $deliveryType . '", ';
				$onlineOrder .= '"paid":false, ';
				$onlineOrder .= '"orderTime":' . $time . ', ';
				$onlineOrder .= '"clientId":' . $clientId . ', ';
				$onlineOrder .= '"address":"' . $lk->user_address . '", ';
				$onlineOrder .= '"responsiblePersonLogin":"' . $login . '", ';
				$onlineOrder .=
					'"totalAmount":' .
					self::__formatprc($lk->order_price) .
					', ';
				$onlineOrder .= '"currentStatusType":"' . $statusType . '", ';
				$onlineOrder .= '"currentStatusTime":0';
				$onlineOrder .= '}';

				$data = '{';
				$data .= '"onlineOrder":' . $onlineOrder . ', ';
				$data .= '"onlineOrderItems":[' . $onlineOrderItems . '],';
				$data .= '"onlineOrderPayments":[' . $onlineOrderPayments . ']';
				$data .= '}';
			}

			// get the ID of the document being processed (id)
			$receipt_id = json_decode(
				self::__loadData($url, $headers, $data, $type)
			);

			// error conditions
			if (is_object($receipt_id) && $receipt_id->type == 'ERROR') {
				$mess =
					_wp('Связь. Ошибка') . ': ' . $receipt_id->message . '!';
			} else {
				$mess = $is_update
					? _wp('Связь. Успешно удалён заказ №') . $lk->order_number
					: _wp('Связь. Успешно передан заказ №') .
						$lk->order_number .
						'. ' .
						_wp('Номер документа:') .
						$receipt_id;
			}

			// write log
			self::writeLog($mess);
		}
	}

	function __mainProcess($order, $test, $token, $i)
	{
		// 1. AUTHORIZATION
		//    process necessary to connect a client to fiscal processing

		// address for POST request
		$url = 'https://fp' . $test . '.bifit.com/processing-api/oauth/token';

		// link data
		$data = 'token=' . $token;
		$data .= '&client_id=processing-connector-token';
		$data .= '&client_secret=processing-connector-token';
		$data .= '&grant_type=token';

		// headers
		$headers = array('Content-Type: application/x-www-form-urlencoded');

		// decode the received json response
		$bifit = json_decode(self::__loadData($url, $headers, $data));

		// 2. FORMATION OF THE CONTENT OF A CHECK
		//    formation of information for further sending for fiscalization

		// Plugin parameters

		// cashier name
		$cashier_name = $this->getSettings('cashier_name');

		// TIN of the cashier
		$cashier_inn = $this->getSettings('cashier_inn');

		// document type
		$arr_rec_type = array(
			'1' => 'SALE',
			'2' => 'SALE_RETURN',
			'3' => 'PURCHASE',
			'4' => 'PURCHASE_RETURN'
		);
		$receipt_type = $arr_rec_type[$i];

		// tax_system
		$tax_system = $this->getSettings('tax_system_' . $i);

		// method of calculation
		$calc_method = $this->getSettings('calculation_method');

		// VAT
		$vat = $this->getSettings('vat');

		// payment type
		$payment_type = $this->getSettings('payment_type');

		// place of settlement
		$payment_address = $this->getSettings('payment_address');

		// client address type
		$address_type = $this->getSettings('address_type');

		// full name of the client
		$fio = $this->getSettings('fio');

		// user data
		$user_firstname = $order->contact->get('firstname', 'default');
		$user_lastname = $order->contact->get('lastname', 'default');
		$user_name = trim($user_lastname . ' ' . $user_firstname);
		$user_phone = self::__checkPhone(
			$order->contact->get('phone', 'default')
		);
		$user_email = self::__checkEmail(
			$order->contact->get('email', 'default')
		);
		$user_address = '';
		$address_arr = ['street', 'city', 'region', 'zip', 'country'];
		foreach ($address_arr as $val) {
			$addr = $order->contact->get('address:' . $val, 'default');
			if ($addr != '') {
				$user_address .= $user_address != '' ? ', ' . $addr : $addr;
			}
		}

		// buyer's address on the receipt (phone or email)
		$address = $address_type == 'phone' ? $user_phone : $user_email;

		// Array parameters

		/// subtotal
		$o_subtotal = 0;
		/// an array of products to fill
		$r_products = [];
		/// an array of order items
		$products = $order->items;
		/// shipping price
		$o_shipment = $order->shipping;

		// iterate over the array of order items
		foreach ($products as $key_id => $product) {
			/// adjusting the price of the product taking into account discounts
			$p_price = self::__formatprc(
				self::__checkProductPrice(
					$product['price'],
					$order->total + $order->total,
					$order->discount,
					0
				)
			);

			/// we form a subtotal
			$o_subtotal += $p_price * $product['quantity'];

			// we clear the name of the product from special characters
			$p_name = preg_replace(
				'/[^\p{L}0-9 ]/iu',
				' ',
				html_entity_decode($product['name'])
			);
			$p_name = preg_replace('/ +/', ' ', trim($p_name));

			// form an array of goods for the check
			$r_products[$key_id] = new stdClass();
			$r_products[$key_id]->product_id = $product['product_id'];
			$r_products[$key_id]->product_name = $p_name;
			$r_products[$key_id]->product_item_price = $p_price;
			$r_products[$key_id]->product_quantity = $product['quantity'];
			$r_products[$key_id]->product_tax = 0;
		}

		// check parameters
		$items = '';

		// generate check data
		foreach ($r_products as $key_id => $product) {
			// display the product data in the check
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .= '"calculationMethod":"' . $calc_method . '", ';
			$items .= '"paymentSubject":"PRODUCT", ';
			$items .= '"name":"' . $product->product_name . '", ';
			$items .= '"price":' . $product->product_item_price . ', ';
			$items .= '"quantity":' . $product->product_quantity . ', ';
			$items .= '"vat":"' . $vat . '", ';
			$items .=
				'"total":' .
				self::__formatprc($p_price * $product->product_quantity);
			$items .= '}';
		}

		// delivery
		if ($o_shipment > 0) {
			// display the data of the delivery method in the check
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .= '"calculationMethod":"' . $calc_method . '", ';
			$items .= '"paymentSubject":"SERVICE", ';
			$items .= '"name":"' . _wp('Доставка') . '", ';
			$items .= '"price":' . self::__formatprc($o_shipment) . ', ';
			$items .= '"quantity":1, ';
			$items .= '"vat":"' . $vat . '", ';
			$items .= '"total":' . self::__formatprc($o_shipment);
			$items .= '}';
		}

		// form a check
		$receipt = '{';
		$receipt .=
			'"type":"' .
			$receipt_type .
			'", "taxSystem":"' .
			$tax_system .
			'", ';
		$receipt .= '"cashier":{';
		$receipt .= '"name":"' . $cashier_name . '", ';
		$receipt .= '"inn":"' . self::__checkInn($cashier_inn, 1) . '"';
		$receipt .= '}, ';
		$receipt .= '"client":{';
		$receipt .= $fio ? '"name":"' . $user_name . '", ' : '';
		$receipt .= '"address":"' . $address . '"';
		$receipt .= '}, ';
		$receipt .= '"paymentAddress":"' . $payment_address . '", ';
		$receipt .= '"items":[' . $items . '], ';
		$receipt .= '"total":' . self::__formatprc($order->total) . ', ';
		$receipt .=
			'"payments":{"' .
			$payment_type .
			'": ' .
			self::__formatprc($order->total) .
			'}';
		$receipt .= '}';

		// 3. SENDING A CHECK TO FISCAL PROCESSING
		//    transfer of the content of the check to fiscal processing for further fiscalization

		// address for POST request
		$url =
			'https://fp' .
			$test .
			'.bifit.com/processing-api/protected/documents/registration/receipts';

		// check data
		$data = $receipt;

		// Idempotency-Key
		$o = [];
		$o['number'] = $order->id;
		$o['summ'] = self::__formatprc($order->total);
		$o['cms'] = 'Shop-Script';
		$idemp_key = self::__createGUID($o);

		// headers
		$headers = array(
			'Content-Type: application/json',
			'Authorization: Bearer ' . $bifit->access_token,
			'Idempotency-Key: ' . $idemp_key
		);

		// get the ID of the document being processed (id)
		$receipt_id = json_decode(self::__loadData($url, $headers, $data));

		// 4. RECEIVING DOCUMENT BY ID
		//    obtaining a document with fiscal characteristics by its number

		// options
		$doc = '';

		if (!is_object($receipt_id)) {
			// link for GET request
			$url =
				'https://fp' .
				$test .
				'.bifit.com/processing-api/protected/documents/' .
				$receipt_id;

			// headers
			$headers = array('Authorization: Bearer ' . $bifit->access_token);

			// we get the document
			$doc = json_decode(self::__loadData($url, $headers, false, 'GET'));
		}

		// LINK TO THE CHECK

		// we form a link to the check
		$receipt_link = !is_object($receipt_id)
			? 'https://fp' .
				$test .
				'.bifit.com/processing-api/receipts/' .
				$idemp_key
			: 'XXX';

		// OPERATION HISTORY

		// object parameters
		$object = new stdClass();
		/// order id
		$object->order_id = $order->id;
		/// order number
		$object->order_number = $order->id_str;
		/// order date
		$object->order_date = $order->create_datetime;
		/// date of operation
		$object->action_date = waDateTime::date('Y-m-d H:i:s');
		/// link to check
		$object->receipt_link = $receipt_link;
		/// id of the received document
		$object->receipt_id = !is_object($receipt_id) ? $receipt_id : '0';
		/// type of document being held
		$object->receipt_type = $receipt_type;
		/// operation log (to be filled in later)
		$object->log = '';

		// save the operation to history
		self::__saveReport(
			$test,
			$bifit,
			$receipt,
			$receipt_id,
			$doc,
			$object,
			$idemp_key
		);
	}

	function __formatprc($price)
	{
		$dec_count = 2;
		$price =
			$dec_count < 0
				? round($price, $dec_count)
				: number_format($price, $dec_count, '.', '');

		return $price;
	}

	function __checkInn($inn, $type = 1)
	{
		if (isset($inn) && trim($inn) != '') {
			if ($type == 2) {
				$inn = !preg_match("/^\d{10}$/", trim($inn))
					? _wp('ИНН задан в неверном формате!')
					: trim($inn);
			} elseif ($type == 1) {
				$inn = !preg_match("/^\d{12}$/", trim($inn))
					? _wp('ИНН задан в неверном формате!')
					: trim($inn);
			} else {
				$inn = !preg_match("/^\d{10}$|^\d{12}$/", trim($inn))
					? _wp('ИНН задан в неверном формате!')
					: trim($inn);
			}
		} else {
			$inn = _wp('ИНН пуст или не найден!');
		}

		return $inn;
	}

	function __checkPhone($tel)
	{
		if (isset($tel) && trim($tel) != '') {
			$phone = str_replace(
				array('+', '(', ')', '-', ' '),
				'',
				trim($tel)
			);
			$first = substr($phone, "0", 1);
			$phone =
				strlen($phone) == 10 && $first != 7 ? '7' . $phone : $phone;
			$phone = !preg_match("/^[0-9]{11}+$/", $phone)
				? _wp('Телефон задан в неверном формате!')
				: $phone;
		} else {
			$phone = _wp('Телефон пуст или не найден!');
		}

		return $phone;
	}

	function __checkEmail($email)
	{
		$email = !preg_match("/^\+\d+$|^\S+@\S+$/", $email)
			? _wp('E-mail задан в неверном формате!')
			: $email;

		return $email;
	}

	function __checkProductPrice(
		$item_price,
		$order_subtotal,
		$order_discount = 0,
		$order_payment = 0
	) {
		if ($order_discount > 0) {
			$percent = ($order_discount * 100) / $order_subtotal;
			$item_price = $item_price - $item_price * ($percent / 100);
		}

		if ($order_payment > 0) {
			$percent =
				($order_payment * 100) / ($order_subtotal - $order_discount);

			$item_price = $item_price + $item_price * ($percent / 100);
		}

		return $item_price;
	}

	function __checkResponce($response)
	{
		$mess = '';

		if (
			is_object($response) &&
			($response->type == 'ERROR' || $response->status == 400)
		) {
			$mess = '';
			if ($response->status == 400) {
				$mess .= $response->error . "\n";
				$mess .= $response->message . "\n";
			} else {
				$array = json_decode(json_encode($response), true);
				array_walk_recursive($array, function ($item, $key) use (
					&$mess
				) {
					if ($item != 'ERROR' && trim($item != '')) {
						$mess .= $key . ': ' . $item . "\n";
					}
				});
			}
		} elseif (is_object($response)) {
			$array = json_decode(json_encode($response), true);
			array_walk_recursive($array, function ($item, $key) use (&$mess) {
				if (trim($item != '')) {
					$mess .= $key . ': ' . $item . "\n";
				}
			});
		}

		return $mess;
	}

	function __loadData($url, $headers = false, $data = false, $type = 'POST')
	{
		$ch = curl_init($url);

		if ($type != 'POST') {
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $type);
		} else {
			curl_setopt($ch, CURLOPT_POST, true);
		}

		if ($data) {
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		}

		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_ENCODING, '');
		curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
		curl_setopt($ch, CURLOPT_TIMEOUT, 0);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);

		if ($headers) {
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		}

		$output = curl_exec($ch);
		curl_close($ch);

		return $output;
	}

	function __getToken($login, $password)
	{
		$signature = urlencode(base64_encode(hash('sha256', $password, true)));
		$data = 'username=' . $login;
		$data .= '&password=' . $signature;
		$data .= '&client_id=cashdesk-rest-client';
		$data .= '&client_secret=cashdesk-rest-client';
		$data .= '&grant_type=password';
		$url = 'https://kassa.bifit.com/cashdesk-api/v1/oauth/token';
		$headers = array('Content-Type: application/x-www-form-urlencoded');
		$tokenData = json_decode(self::__loadData($url, $headers, $data));

		return $tokenData;
	}

	function __getOrganizations($token)
	{
		$url =
			'https://kassa.bifit.com/cashdesk-api/v1/protected/organizations/list/read';
		$headers = array('Authorization: Bearer ' . $token);
		$orgData = json_decode(self::__loadData($url, $headers));

		return $orgData;
	}

	function __getTradeObjects($org_id, $token)
	{
		$data = 'organization_id=' . $org_id;
		$url =
			'https://kassa.bifit.com/cashdesk-api/v1/protected/trade_objects/list/read';
		$headers = array(
			'Content-Type: application/x-www-form-urlencoded',
			'Authorization: Bearer ' . $token
		);
		$tradeData = json_decode(self::__loadData($url, $headers, $data));

		return $tradeData;
	}

	function __createMainArray($orgData, $token)
	{
		$arr = [];

		foreach ($orgData as $key => $org) {
			$arr[$key] = new stdClass();
			$arr[$key]->id = $org->id;
			$arr[$key]->name = $org->name;
			$arr[$key]->traides = [];

			$tradeData = self::__getTradeObjects($org->id, $token);

			foreach ($tradeData as $k => $trd) {
				$arr[$key]->traides[$k] = new stdClass();
				$arr[$key]->traides[$k]->id = $trd->id;
				$arr[$key]->traides[$k]->name = $trd->name;
			}
		}

		return $arr;
	}

	function __createSelects($name)
	{
		$plugin_id = self::PLUGIN_ID;
		$plugin = waSystem::getInstance()->getPlugin($plugin_id, true);
		$organizations = is_array($plugin->getSettings('organizations'))
			? $plugin->getSettings('organizations')
			: [];
		$data = [];

		if (is_array($organizations) && count($organizations)) {
			$org_id =
				$plugin->getSettings('organization') != -1 &&
				$plugin->getSettings('organization') != 1
					? $plugin->getSettings('organization')
					: $organizations[0]['id'];

			foreach ($organizations as $key => $org) {
				if ($name == 'organization') {
					$data[] = [
						'value' => $org['id'],
						'title' => $org['name']
					];
				}

				if ($name == 'trade_object' && $org['id'] != $org_id) {
					continue;
				}

				if ($name != 'organization') {
					foreach ($org['traides'] as $k => $trd) {
						$data[] = [
							'value' => $trd['id'],
							'title' => $trd['name']
						];
					}
				}
			}
		}

		return $data;
	}

	function __saveReport(
		$test,
		$bifit,
		$receipt,
		$receipt_id,
		$doc,
		$object,
		$idemp_key
	) {
		// display the property of the operation
		$log =
			$test != ''
				? '####### ' . _wp('Тестовый запрос') . "\n"
				: '####### ' . _wp('Боевой запрос') . "\n";

		// set the time zone according to the site settings
		$log .= '####### ' . date('Y-m-d H:i:s', time()) . "\n";

		// ANSWER #1

		$log .= '####### ' . _wp('Ответ') . ' #1: ' . "\n";

		// response data # 1
		$token_error = null;

		if (is_object($bifit)) {
			foreach ($bifit as $key => $val) {
				if ($key == 'error') {
					$token_error = 1;
				}
				$log .= $key . ': ' . $val . "\n";
			}
		}

		// RECEIPT

		// generated receipt for sending
		$log .= '####### ' . _wp('Чек') . ': ' . "\n" . $receipt . "\n";

		if (!$token_error) {
			// IDEMPOTENCY KEY

			$log .=
				'####### ' .
				_wp('Ключ идемпотентности') .
				': ' .
				"\n" .
				$idemp_key .
				"\n";

			// ANSWER #2

			$log .= '####### ' . _wp('Ответ') . ' #2: ' . "\n";

			// response data #2
			if (is_object($receipt_id)) {
				$log .= self::__checkResponce($receipt_id);
			} else {
				$log .= 'Номер документа: ' . $receipt_id . "\n";

				// ANSWER #3

				$log .= '####### ' . _wp('Ответ') . ' #3: ' . "\n";

				if (is_object($doc)) {
					$log .= self::__checkResponce($doc);
				}

				// LINK TO THE CHECK

				// generated link to the check
				$log .=
					'####### ' .
					_wp('Ссылка на чек') .
					': ' .
					"\n" .
					$object->receipt_link;
			}
		}

		$object->log = $log;

		// SAVE

		self::writeLog($log);

		// object params
		$object->log = $log;

		$log_model = new shopOrderLogModel();
		$log_model->add([
			'order_id' => $object->order_id,
			'action_id' => '',
			'contact_id' => null,
			'before_state_id' => "new",
			'after_state_id' => "new",
			'text' => _wp('Ссылка на чек') . ': ' . "\n" . $object->receipt_link
		]);
	}

	public function writeLog($message)
	{
		if (is_string($message)) {
			waLog::log($message, self::LOG_FILE_NAME);
		} else {
			waLog::dump($message, self::LOG_FILE_NAME);
		}
	}

	/**
	 * @param $order_id
	 * @return shopOrder|false
	 */
	private function getOrder($order_id)
	{
		try {
			$order = new shopOrder($order_id);
		} catch (Exception $e) {
			waLog::log('Order not found: ' . $order_id, self::LOG_FILE);

			return false;
		}

		return $order;
	}

	public function addressTypeValues()
	{
		$res = array(
			array(
				'value' => 'phone',
				'title' => 'Телефон'
			),
			array(
				'value' => 'email',
				'title' => 'E-mail'
			)
		);

		return $res;
	}

	public function receiptTypeValues()
	{
		$res = array(
			array(
				'value' => 'SALE',
				'title' => 'Приход'
			),
			array(
				'value' => 'SALE_RETURN',
				'title' => 'Возврат прихода'
			),
			array(
				'value' => 'PURCHASE',
				'title' => 'Расход'
			),
			array(
				'value' => 'PURCHASE_RETURN',
				'title' => 'Возврат расхода'
			)
		);

		return $res;
	}

	public function taxSystemValues()
	{
		$res = array(
			array(
				'value' => 'COMMON',
				'title' => 'ОСН'
			),
			array(
				'value' => 'SIMPLIFIED',
				'title' => 'УСН доход'
			),
			array(
				'value' => 'SIMPLIFIED_WITH_EXPENS',
				'title' => 'УСН доход-расход'
			),
			array(
				'value' => 'ENVD',
				'title' => 'ЕНВД'
			),
			array(
				'value' => 'COMMON_AGRICULTURAL',
				'title' => 'ЕСХН'
			),
			array(
				'value' => 'PATENT',
				'title' => 'ПАТЕНТ'
			)
		);

		return $res;
	}

	public function calculationMethodValues()
	{
		$res = array(
			array(
				'value' => 'PREPAY_FULL',
				'title' =>
					'Полная предварительная оплата до момента передачи предмета расчёта'
			),
			array(
				'value' => 'PREPAY_PARTIAL',
				'title' =>
					'Частичная предварительная оплата до момента передачи предмета расчёта'
			),
			array(
				'value' => 'AVANS',
				'title' => 'Аванс'
			),
			array(
				'value' => 'FULL_PAY',
				'title' =>
					'Полная оплата, в том числе с учётом аванса (предварительной оплаты) в момент передачи предмета расчёта'
			),
			array(
				'value' => 'PARTIAL_SETTLEMENT_AND_CREDIT',
				'title' =>
					'Частичная оплата предмета расчёта в момент его передачи с последующей оплатой в кредит'
			),
			array(
				'value' => 'TRANSFER_ON_CREDIT',
				'title' =>
					'Передача предмета расчёта без его оплаты в момент его передачи с последующей оплатой в кредит'
			),
			array(
				'value' => 'CREDIT_PAYMENT',
				'title' =>
					'Оплата предмета расчёта после его передачи с оплатой в кредит (оплата кредита)'
			)
		);

		return $res;
	}

	public function paymentTypeValues()
	{
		$res = array(
			array(
				'value' => 'CASH',
				'title' => 'Сумма оплаты наличными'
			),
			array(
				'value' => 'CARD',
				'title' => 'Сумма оплата безналичными'
			),
			array(
				'value' => 'PREPAY',
				'title' => 'Сумма оплаты предоплатой (зачётом аванса)'
			),
			array(
				'value' => 'POSTPAY',
				'title' => 'Сумма оплаты постоплатой (в кредит)'
			),
			array(
				'value' => 'OTHER',
				'title' => 'Сумма оплаты встречным предоставлением'
			)
		);

		return $res;
	}

	public function vatValues()
	{
		$res = array(
			array(
				'value' => 'VAT_0',
				'title' => 'НДС 0%'
			),
			array(
				'value' => 'VAT_10',
				'title' => 'НДС 10%'
			),
			array(
				'value' => 'VAT_20',
				'title' => 'НДС 20%'
			),
			array(
				'value' => 'VAT_110',
				'title' => 'НДС 10/110'
			),
			array(
				'value' => 'VAT_120',
				'title' => 'НДС 20/120'
			)
		);

		return $res;
	}

	public function orderPaidValues()
	{
		$res = array(
			array(
				'value' => 0,
				'title' => 'Не учитывать'
			),
			array(
				'value' => 1,
				'title' => 'Нет'
			),
			array(
				'value' => 2,
				'title' => 'Да'
			)
		);

		return $res;
	}

	/**
	 * Возвращает список возможных действий для заказа
	 *
	 * @return array
	 */
	public static function settingAvailableOrderActions()
	{
		$workflow = new shopWorkflow();
		$actions = $workflow->getAvailableActions();
		$data = [];

		foreach ($actions as $key => $action) {
			if (isset($action['state'])) {
				$data[] = [
					'value' => $key,
					'title' => $action['name']
				];
			}
		}

		return $data;
	}

	public static function getDeliveryMethods()
	{
		$spm = new shopPluginModel();
		$methods = $spm->listPlugins('shipping');
		$data = [];

		foreach ($methods as $key => $method) {
			$data[] = [
				'value' => $method['id'],
				'title' => $method['name']
			];
		}

		return $data;
	}

	public static function getPaymentTypes()
	{
		$spm = new shopPluginModel();
		$methods = $spm->listPlugins('payment');
		$data = [];

		foreach ($methods as $k => $v) {
			$data[] = [
				'value' => $v['id'],
				'title' => $v['name']
			];
		}

		return $data;
	}

	public static function getOrganizationsSelect()
	{
		$orgs_sel = self::__createSelects('organization');

		return $orgs_sel;
	}

	public static function getTradeObjectsSelect()
	{
		$trade_objects_sel = self::__createSelects('trade_object');

		return $trade_objects_sel;
	}

	public static function getOrganizationsTextarea()
	{
		$plugin_id = self::PLUGIN_ID;
		$plugin = waSystem::getInstance()->getPlugin($plugin_id, true);
		$login = self::__checkPhone($plugin->getSettings('login'));
		$password = $plugin->getSettings('password');
		$organizations = is_array($plugin->getSettings('organizations'))
			? $plugin->getSettings('organizations')
			: [];

		if (!count($organizations) && $login && $password) {
			$tokenData = self::__getToken($login, $password);
			$token = $tokenData->access_token;
			$orgData = self::__getOrganizations($token);
			$organizations = self::__createMainArray($orgData, $token);
		}

		$controls =
			'<textarea id="bifitkassa_shop_bifitkassa_organizations" name="shop_bifitkassa[organizations]" style="display:none;">' .
			json_encode($organizations) .
			'</textarea>';

		$controls .= '<style rype="text/css">';
		$controls .= '.block.double-padded>h1{position:relative;}';
		$controls .=
			'.block.double-padded>h1::after{content:\'Касса\';position:absolute;top:0;left:98px;color:#189AF7;background:#FFF;}';
		$controls .= '.field>.name{text-align:right;}';
		$controls .=
			'.field .value input[type="text"], .field .value input[type="password"], .field .value textarea, .field .value select{border:1px solid #AAA;padding:3px;}';
		$controls .= '.field .value select{min-width:223px;}';
		$controls .= '</style>';
		$controls .= '<script type="text/javascript">';
		$controls .= <<<JS
		function updateLK(){jQuery("#bifitkassa_shop_bifitkassa_organizations").html(""),jQuery("#plugins-settings-form").submit(),location.reload()}function updateTraides(t,s){var o=[],i=jQuery("#bifitkassa_shop_bifitkassa_organizations").html();organizations=jQuery.parseJSON(i),jQuery.each(organizations,function(i,a){if(a.id===t)return jQuery.each(a.traides,function(i,a){o.push("<option value="+a.id+">"+a.name+"</option>")}),jQuery("#bifitkassa_shop_bifitkassa_trade_object").html(o.join("")),s&&jQuery("#up1").text(s),!1})}function updateOrgs(){var t=[],i=jQuery("#bifitkassa_shop_bifitkassa_organizations").html();organizations=jQuery.parseJSON(i),jQuery.each(organizations,function(i,a){t.push("<option value="+a.id+">"+a.name+"</option>")}),jQuery("#bifitkassa_shop_bifitkassa_organization").html(t.join("")),updateTraides(jQuery("#bifitkassa_shop_bifitkassa_organization").find("option").eq(0).val(),"Обновлено!"),jQuery("body,html").animate({scrollTop:jQuery("#up1").offset().top-100},"slow")}jQuery(document).on("change","#bifitkassa_shop_bifitkassa_organization",function(){updateTraides(jQuery(this).val())}),jQuery(document).ready(function(){jQuery("#bifitkassa_shop_bifitkassa_trade_object").has("option").length||(updateOrgs(),jQuery("#plugins-settings-form").submit())});
		JS;
		$controls .= '</script>';

		return $controls;
	}
}